;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "hare-mode" "20260316.1559"
  "Hare mode."
  '((emacs "25.1"))
  :url "https://git.sr.ht/~grafov/hare-mode"
  :commit "7c757157502d916932bbc7668e1606a67e6b3b22"
  :revdesc "7c757157502d"
  :keywords '("languages")
  :authors '(("Benjamín Buccianti" . "benjamin@buccianti.dev")
             ("Amin Bandali" . "bandali@gnu.org")
             ("Theodor Thornhill" . "theo@thornhill.no")
             ("Sebastian" . "sebastian@sebsite.pw")
             ("Alexander Grafov" . "grafov@inet.name"))
  :maintainers '(("Benjamín Buccianti" . "benjamin@buccianti.dev")
                 ("Amin Bandali" . "bandali@gnu.org")
                 ("Theodor Thornhill" . "theo@thornhill.no")
                 ("Sebastian" . "sebastian@sebsite.pw")
                 ("Alexander Grafov" . "grafov@inet.name")))
